/**
 *
 * Copyright (C) 2009-2014 INRA.
 * Copyright (C) 2009-2014 CETIOM.
 *
 *
 * TODO licence
 */

// @@tagdynamic@@
// @@tagdepends: vle.discrete-time @@endtagdepends


#include <math.h>
#include <vle/DiscreteTime.hpp>
#include <sunflo_utils.hpp>
#include <Phenologie.hpp>
#include <ParametresVariete.hpp>

namespace sunflo {

using namespace vle::discrete_time;

class ElaborationQualite : public DiscreteTimeDyn
{
public :

    ParametresVariete pv;
   // double densite;

    /*Sync*/ Var PhasePhenoPlante;


    /// # Jours entre maturation et récolte.
    Var D_MH;
   /// # Jours entre semis et germination.
    Var D_SE;

    /// # Moyenne ETPET entre maturation et récolte.
    Var MET_MH;

    /// # Moyenne ETPET durant les jours de phase juvénile et croissance active.
    Var MET_EF;

    // Thermal stress entre germinatioan et initiation florale.
    Var NHT28_EF;
 
    // Thermal stress during whole season
    Var NHT28;
    // Sum of radiations during flowering
     Var SGR_FM;


    /// Teneur en huile (0%), modèle linéaire [Casadebaig2008]
    Var TH;
    /// Capture de TH a un instant donne
    Var photo_TH_aFinMATURATION;




    ElaborationQualite(const vle::devs::DynamicsInit& model,
            const vle::devs::InitEventList& events) :
                DiscreteTimeDyn(model, events)
    {
        pv.initialiser(events);

        PhasePhenoPlante.init(this, "PhasePhenoPlante", events);


        TH.init(this, "TH" , events);

        photo_TH_aFinMATURATION.init(this, "photo_TH_aFinMATURATION" , events);

	D_MH.init(this, "D_MH" , events);
	D_SE.init(this, "D_SE", events);
        MET_MH.init(this, "MET_MH" , events);
        MET_EF.init(this, "MET_EF" , events);
	NHT28_EF.init(this, "NHT28_EF", events);
	NHT28.init(this, "NHT28", events);
	SGR_FM.init(this, "SGR_FM", events);
    }

    virtual ~ElaborationQualite() { }

    virtual void compute(const vle::devs::Time& /*time*/)
    {

            {
                double TH_tmp = 0.0;
                if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_FLORAISON )
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    TH_tmp = 0.0;
                } else {
// Add LLH 
                    TH_tmp = 9.43 - 0.130*D_SE() + 0.131*D_MH() -
		    0.00911*SGR_FM()  -12.8*MET_EF() + 2.54*MET_MH() -
		    0.209*NHT28() - 0.882*NHT28_EF() + 1.02*pv.thp - 0.201*pv.bSF;
                }
                TH = TH_tmp; 

                if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_MATURATION) and
                        (PhasePhenoPlante() == PHASEPHENOPLANTE_DESSICATION)){
                    photo_TH_aFinMATURATION = TH(-1);
                } else if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_MATURATION) and
                        (PhasePhenoPlante() == PHASEPHENOPLANTE_RECOLTEE)){
                    photo_TH_aFinMATURATION = TH(-1);
                } else {
                    photo_TH_aFinMATURATION = photo_TH_aFinMATURATION(-1);
                }
            }
    }
};
} // namespace sunflo

DECLARE_DYNAMICS(sunflo::ElaborationQualite);

