/**
 *
 * Copyright (C) 2009-2014 INRA.
 * Copyright (C) 2009-2014 CETIOM.
 *
 *
 * TODO licence
 */

// @@tagdynamic@@
// @@tagdepends: vle.discrete-time @@endtagdepends


#include <math.h>
#include <vle/DiscreteTime.hpp>
#include <sunflo_utils.hpp>
#include <Phenologie.hpp>
#include <ParametresVariete.hpp>
#include <ParametresRendement.hpp>


namespace sunflo {

using namespace vle::discrete_time;

class ElaborationRendement : public DiscreteTimeDyn
{
public :
    ParametresRendement pr;
    ParametresVariete pv;
    bool first_compute;

    /*Sync*/ Var PhasePhenoPlante;
    /*Sync*/ Var TDM;
   // /*Sync*/ Var INN;

    /// # Rendement (0% humidité, 0% impuretés)
    Var RDT;
    /// # Modèle linéaire expliquant l’indice de récolte
    Var IRs;
    /// Capture de TDM a un instant donne (ex TDMF1)
    Var photo_TDM_CROISSANCEACTIVE_A_FLORAISON;
    /// Capture de IRs a un instant donne (ex IR)
    Var photo_IRs_aFinMATURATION;
    /// Capture de RDT a un instant donne
    Var photo_RDT_aFinMATURATION;


// Covariables to compute GY

// Somme précipitations durant la phase végétative (covariables statistiques)
    Var SRR_EF;
// Somme précipitations durant la floraison  (covariables statistiques)
    Var SRR_FM;
// Somme radiations interceptées durant la phase végétative (covariables statistiques)
    Var SIR_EF;
// Somme  radiations interceptées durant la floraison  (covariables statistiques)
    Var SIR_FM;
// Effet contrainte hydrique sur la photosynthèse durant la phase végétative (covariables statistiques)
    Var SFHTR_EF;
//  Effet contrainte hydrique sur la photosynthèse durant la floraison  (covariables statistiques)
    Var SFHTR_FM;

//  Effet contrainte azote  sur la photosynthèse après la maturation  (covariables statistiques)
    Var SFNRUE_MH;

// Somme hautes températures durant la floraison
    Var SHT_FM; 


    ElaborationRendement(const vle::devs::DynamicsInit& model,
            const vle::devs::InitEventList& events) :
                DiscreteTimeDyn(model, events)
    {

        pr.initialiser( events );
        pv.initialiser( events );


        PhasePhenoPlante.init(this, "PhasePhenoPlante", events);
        TDM.init(this, "TDM", events);
        RDT.init(this, "RDT" , events);
        IRs.init(this, "IRs" , events);
        photo_TDM_CROISSANCEACTIVE_A_FLORAISON.init(this, "photo_TDM_CROISSANCEACTIVE_A_FLORAISON" , events);
        photo_IRs_aFinMATURATION.init(this, "photo_IRs_aFinMATURATION" , events);
        photo_RDT_aFinMATURATION.init(this, "photo_RDT_aFinMATURATION" , events);
        first_compute = true;

////// Covariables GY
	SRR_EF.init(this, "SRR_EF", events);
	SRR_FM.init(this, "SRR_FM", events);

	SIR_EF.init(this, "SIR_EF", events);
	SIR_FM.init(this, "SIR_FM", events);

	SFHTR_EF.init(this, "SFHTR_EF", events);
	SFHTR_FM.init(this, "SFHTR_FM", events);

	SFNRUE_MH.init(this, "SFNRUE_MH", events);
	SHT_FM.init(this, "SHT_FM", events);
    }

    virtual ~ElaborationRendement() { }

    /**********************************************************************//**
     *
     * Traitement effectue a chaque pas de temps.
     *
     * Description de la facon dont le code ModelMaker de depart a ete interprete, traduit sous VLE : voir TRADUCTION_DE_MODELMAKER_EN_VLE dans la classe de documentation DOCsunfloV1.
     *
     **************************************************************************/
    virtual void compute(const vle::devs::Time& /*time*/)
    {
        if (first_compute) {
            first_compute = false;
            {
                double RDT_tmp = 0.0;
                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_DESSICATION )
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 >= date_TT_M3
                    RDT_tmp = TDM() * photo_IRs_aFinMATURATION() / 10.0;
                } else {
                    RDT_tmp = TDM() * IRs() / 10.0;
                }
                RDT = RDT_tmp;
            }
            photo_RDT_aFinMATURATION = 0.0; 
        } else {

            if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_CROISSANCEACTIVE) and
                    (PhasePhenoPlante() == PHASEPHENOPLANTE_FLORAISON)){
                //photo_TDM_CROISSANCEACTIVE_A_FLORAISON = TDM(-1); // derniere valeur en CROISSANCEACTIVE
                photo_TDM_CROISSANCEACTIVE_A_FLORAISON = TDM(); // première valeur en floraison
            } else if ( PhasePhenoPlante() >= PHASEPHENOPLANTE_FLORAISON ){
                photo_TDM_CROISSANCEACTIVE_A_FLORAISON = photo_TDM_CROISSANCEACTIVE_A_FLORAISON(-1);
            } else {
                photo_TDM_CROISSANCEACTIVE_A_FLORAISON = photo_TDM_CROISSANCEACTIVE_A_FLORAISON(-1);
            }


 /*           {
                double IRs_tmp = 0.0;

                if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_FLORAISON)
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 < date_TT_F1
                    IRs_tmp = 0.0;
                } else {
                    IRs_tmp = (0.416 + (-1.42e-04) * photo_TDM_CROISSANCEACTIVE_A_FLORAISON()
                                    + (1.19e-03) * NET_EF() + (-8.45e-04) * NET_FM()
                                    + (-2.47e-04) * NET_MH() 
                                    + (-3.61e-04) * TT_MH() + 7.75e-01 * pv.IRg);
                }
                IRs = IRs_tmp;
            }
            if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_MATURATION) and
                    (PhasePhenoPlante() == PHASEPHENOPLANTE_DESSICATION)){

                photo_IRs_aFinMATURATION = IRs(-1); // derniere valeur en MATURATION et non pas 1ere en DESSICATION

            } else if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_MATURATION) and
                    (PhasePhenoPlante() == PHASEPHENOPLANTE_RECOLTEE)){

                photo_IRs_aFinMATURATION = IRs(-1); // derniere valeur en MATURATION et non pas 1ere en RECOLTEE

            } else if ( PhasePhenoPlante() >= PHASEPHENOPLANTE_DESSICATION ){
                photo_IRs_aFinMATURATION = photo_IRs_aFinMATURATION(-1);
            } else {
                photo_IRs_aFinMATURATION = photo_IRs_aFinMATURATION(-1);
            } */

            {
                double RDT_tmp = 0.0;

                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_DESSICATION )
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
		RDT_tmp =-16.12 + 58.35*pv.IRg + (-0.036)*SRR_EF() +
	 	(-0.062)*SRR_FM() + 0.020*SIR_EF() + 0.075*SIR_FM() +
	 	0.37*SFHTR_EF() + (-0.77)*SFHTR_FM() + (-69.84)*SHT_FM() + (-0.84)*SFNRUE_MH() + 0.011*pv.date_TT_M3;
                } else {

		RDT_tmp =-16.12 + 58.35*pv.IRg + (-0.036)*SRR_EF() +
	 	(-0.062)*SRR_FM() + 0.020*SIR_EF() + 0.075*SIR_FM() +
	 	0.37*SFHTR_EF() + (-0.77)*SFHTR_FM() + (-69.84)*SHT_FM() + (-0.84)*SFNRUE_MH() + 0.011*pv.date_TT_M3;
		
                }
                RDT = RDT_tmp;

                if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_MATURATION) and
                        (PhasePhenoPlante() == PHASEPHENOPLANTE_DESSICATION)){

                    photo_RDT_aFinMATURATION = RDT(-1); // derniere valeur en MATURATION et non pas 1ere en DESSICATION
                } else if ((PhasePhenoPlante(-1) == PHASEPHENOPLANTE_MATURATION) and
                        (PhasePhenoPlante() == PHASEPHENOPLANTE_RECOLTEE)){

                    photo_RDT_aFinMATURATION = RDT(-1); // derniere valeur en MATURATION et non pas 1ere en RECOLTEE
                } else {
                    photo_RDT_aFinMATURATION = photo_RDT_aFinMATURATION(-1);
                }
            }
        }
    }
};

} // namespace sunflo

DECLARE_DYNAMICS(sunflo::ElaborationRendement);

